@@ -3,14 +3,16 @@ package XML::LibXML::jQuery;
 use 5.008001;
 use strict;
 use warnings;
-use parent qw/Exporter/;
+use Exporter qw(import);
 use Scalar::Util qw/ blessed /;
 use XML::LibXML;
 use HTML::Selector::XPath qw/selector_to_xpath/;
 use Carp qw/ confess /;
 use JSON qw/ decode_json /;
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
+
+our @EXPORT_OK = qw/ j fn /;
 our @EXPORT = qw/ j /;
 
 use constant {
@@ -24,9 +26,32 @@ use constant {
 
 our ($PARSER);
 
+# plugin functions
+my %fn;
+
 # for data()
 my $data = {};
 
+sub fn($$) {
+    my ($name, $sub) = @_;
+
+    die sprintf("fn '$name' already defined by %s (at %s line %s)", @{$fn{$name}->{caller}})
+        if exists $fn{$name};
+
+    $fn{$name} = {
+        sub => $sub,
+        caller => [caller]
+    };
+}
+
+
+
+#*j = \&jQuery;
+sub j {
+    __PACKAGE__->new(@_);
+}
+
+
 sub new {
     my ($class, $stuff, $before) = @_;
     my ($self, $existing_document, $nodes);
@@ -38,43 +63,46 @@ sub new {
         $existing_document = $self->{document};
     }
 
-    if (blessed $stuff) {
+    if (defined $stuff) {
 
-        if ($stuff->isa(__PACKAGE__)) {
-            $nodes = $stuff->{nodes};
+        if (blessed $stuff) {
+
+            if ($stuff->isa(__PACKAGE__)) {
+                $nodes = $stuff->{nodes};
+            }
+            elsif ($stuff->isa('XML::LibXML::Element')) {
+                $nodes = [$stuff];
+            }
+            else {
+                confess "can't handle this type of object: ".ref $stuff;
+            }
         }
-        elsif ($stuff->isa('XML::LibXML::Element')) {
-            $nodes = [$stuff];
+        elsif (ref $stuff eq 'ARRAY') {
+
+            $nodes = $stuff;
         }
         else {
-            confess "can't handle this type of object: ".ref $stuff;
+            # parse as string
+            if ($stuff =~ /^\s*<\?xml/) {
+                die "xml parsing disabled!";
+                $nodes = [ $PARSER->parse_string($stuff) ];
+            } else {
+                $nodes = [ _parse_html($stuff) ];
+            }
         }
-    }
-    elsif (ref $stuff eq 'ARRAY') {
 
-        $nodes = $stuff;
-    }
-    else {
-        # parse as string
-        if ($stuff =~ /^\s*<\?xml/) {
-            die "xml parsing disabled!";
-            $nodes = [ $PARSER->parse_string($stuff) ];
-        } else {
-            $nodes = [ _parse_html($stuff) ];
-        }
-    }
+        # catch bugs :)
+        confess "undefined node" if grep { !defined } @$nodes;
 
-    # catch bugs :)
-    confess "undefined node" if grep { !defined } @$nodes;
+        # adopt nodes to existing document
+        if ($existing_document) {
 
-    # adopt nodes to existing document
-    if ($existing_document) {
-
-        # my $doc_id = $existing_document->unique_key;
-        $existing_document->adoptNode($_)
+            # my $doc_id = $existing_document->unique_key;
+            $existing_document->adoptNode($_)
             for grep { not $existing_document->isSameNode($_->ownerDocument) }
-                grep { $_->nodeType != XML_DOCUMENT_NODE }
-                @$nodes;
+            grep { $_->nodeType != XML_DOCUMENT_NODE }
+            @$nodes;
+        }
     }
 
     my $document = defined $nodes->[0] ? $nodes->[0]->ownerDocument
@@ -92,12 +120,6 @@ sub new {
     }, $class;
 }
 
-
-#*j = \&jQuery;
-sub j {
-    __PACKAGE__->new(@_);
-}
-
 sub _parse_html {
     my $source = $_[0];
 
@@ -384,9 +406,9 @@ sub each {
 
     for (my $i = 0; $i < @{$self->{nodes}}; $i++) {
 
-        local $_ = $self->new($self->{nodes}[$i]);
+        local $_ = $self->{nodes}[$i];
         my @rv = $cb->($i, $_);
-        last if @rv == 1 && ! defined $rv[0];
+        last if @rv == 1 && !defined $rv[0];
     }
 
     $self;
@@ -933,6 +955,23 @@ sub _is_selector {
     && $_[0] !~ /<(?!!).*?>/
 }
 
+# TODO rethink this autoload thing... issues:
+# - global var is bad... spooky action at a distance;
+# - and if thats ok, we could just add the subref to the symbol table directly
+
+sub AUTOLOAD {
+    my $self = shift;
+    our $AUTOLOAD;
+    (my $method = $AUTOLOAD) =~ s/.*:://s;
+
+    die sprintf "unknown method '$method'"
+        unless ref $self && exists $fn{$method};
+
+    local $_ = $self;
+    $fn{$method}{sub}->(@_);
+}
+
+
 
 # decrement data ref counter, delete data when counter == 0
 sub DESTROY {
